define([
        'dojo/_base/declare',
        'jimu/BaseWidget',
        'dijit/_WidgetsInTemplateMixin',
        'dgrid/OnDemandGrid',
        'dgrid/Selection',
        'dojo/store/Memory',
        'esri/layers/FeatureLayer',
        'esri/symbols/SimpleFillSymbol',
        'esri/tasks/query',
        'esri/Color',
        'dojo/_base/array',
        'dojo/_base/lang',
        'dijit/form/Button'
    ],
    function(declare, BaseWidget, _WidgetsInTemplateMixin,
        Grid, Selection, Memory, FeatureLayer, SimpleFillSymbol,
        Query, Color, array, lang) {
        var StatesGrid = declare([Grid, Selection]);
        var clazz = declare([BaseWidget, _WidgetsInTemplateMixin], {
            baseClass: 'widget-zoomgrid',
            name: 'ZoomGrid',
            statesLayer: null,
            dgrid: null,

            postCreate: function() {
                this.inherited(arguments);
                var columns = [{
                    label: "", //wasn't able to inject an HTML <div> with image here
                    field: "ObjectID",
                    formatter: lang.hitch(this, this.makeZoomButton)
                }, {
                    label: "State",
                    field: "STATE_NAME"
                }, {
                    label: "Population",
                    field: "POP2000"
                }];

                //restrict sorting to the state name field
                for (var i = 0; i < columns.length; i++) {
                    if (i == 1) {
                        columns[i].sortable = true;
                    } else {
                        columns[i].sortable = false;
                    }
                }

                this.dgrid = new StatesGrid({
                    bufferRows: Infinity,
                    columns: columns
                }, this.grid);

                this.statesLayer = new FeatureLayer("https://sampleserver1.arcgisonline.com/ArcGIS/rest/services/Demographics/ESRI_Census_USA/MapServer/5", {
                    mode: FeatureLayer.MODE_SELECTION,
                    outFields: ["ObjectID", "STATE_NAME", "POP2000"]
                });

                //define a selection symbol
                var highlightSymbol = new SimpleFillSymbol().setColor(new Color([50, 205, 50, 0.25]));
                this.statesLayer.setSelectionSymbol(highlightSymbol);

                this.statesLayer.on("load", lang.hitch(this, function(evt) {
                    var query = new Query();
                    query.where = "1=1";
                    evt.layer.queryFeatures(query, lang.hitch(this, function(featureSet) {
                        var items = array.map(featureSet.features, function(feature) {
                            return feature.attributes;
                        });

                        //idProperty must be set manually if value is something other than 'id'
                        var memStore = new Memory({
                            data: items,
                            idProperty: "ObjectID"
                        });
                        this.dgrid.set("store", memStore);
                        this.dgrid.set("sort", "STATE_NAME");

                        this.dgrid.on(".field-ObjectID:click", lang.hitch(this, function(e) {
                            //retrieve the ObjectId when someone clicks on the magnifying glass
                            if (e.target.alt) {
                                this.zoomRow(e.target.alt);
                            }
                        }));
                    }));
                }));

                this.map.addLayers([this.statesLayer]);
            },

            startup: function() {
                this.inherited(arguments);
            },

            zoomRow: function(id) {
                this.statesLayer.clearSelection();
                var query = new Query();
                query.objectIds = [id];
                this.statesLayer.selectFeatures(query, FeatureLayer.SELECTION_NEW, lang.hitch(this, function(features) {
                    //zoom to the selected feature
                    var stateExtent = features[0].geometry.getExtent().expand(5.0);
                    this.map.setExtent(stateExtent);
                }));
            },

            makeZoomButton: function(id) {
                //set the feature 'id' as the alt value for the image so that it can be used to query below
                var zBtn = "<div data-dojo-type='dijit/form/Button'><img src='" + this.folderUrl + "/images/zoom.png' alt='" + id + "'";
                zBtn = zBtn + " width='18' height='18'></div>";
                return zBtn;
            }
        });
        return clazz;
    });
